# -----------------------------------------------------------------------------
# Show a list of assignments with columns for peak heights from one or more
# spectra.
#
import Tkinter

import sparky
import sputil
import tkutil

# -----------------------------------------------------------------------------
#
class relaxation_dialog(tkutil.Dialog):

  # ---------------------------------------------------------------------------
  #
  def __init__(self):

    self.spectra = ()
    
    tkutil.Dialog.__init__(self, 'Relaxation Peak Heights')

    setup = Tkinter.Frame(self.top)
    setup.pack(side = 'top', anchor = 'w')

    # Choose Spectra

    sbutton = tkutil.checkbutton(self.top, 'Choose Spectra', 1)
    sbutton.button.pack(side = 'top', anchor = 'w')

    sc = sputil.spectrum_choices(self.top)
    sc.frame.pack(side = 'top', anchor = 'w')
    self.spectrum_list = sc.spectrum_names
    sbutton.map_widget(sc.frame)

    # Center new peaks and show volume switches

    cb = tkutil.checkbutton(self.top, 'Show volumes?', 0)
    self.show_volume = cb.variable
    cb.button.pack(side = 'top', anchor = 'w')

    cb = tkutil.checkbutton(self.top, 'Center new peaks?', 0)
    self.center_peaks = cb.variable
    cb.button.pack(side = 'top', anchor = 'w')

    # Peak list

    pl = sputil.assignment_listbox(self.top)
    pl.frame.pack(side = 'top', fill = 'both', expand = 1)
    pl.listbox.bind('<ButtonRelease-1>', pl.select_assignment_cb)
    pl.listbox.bind('<Double-ButtonRelease-1>', pl.goto_assignment_cb)
    pl.listbox.bind('<ButtonPress-1>', pl.set_focus_cb)
    pl.listbox.bind('<KeyPress-Delete>', pl.delete_selected_cb)
    self.peak_list = pl

    br = tkutil.button_row(self.top,
                           ('Save', self.peak_list.save_cb),
                           ('Append', self.peak_list.append_cb),
                           ('Clear', self.clear_cb),
			   ('Close', self.close_cb),
                           ('Help', sputil.help_cb('RelaxHts')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def clear_cb(self):

    self.spectra = self.get_spectra()
    self.peak_list.clear()

    heading = '%15s ' % 'assignment'
    for s in self.spectra:
      heading = heading + '%12s' % s.name
    self.peak_list.heading['text'] = heading

  # ---------------------------------------------------------------------------
  #
  def get_spectra(self):

    return map(sputil.name_to_spectrum, self.spectrum_list)
    
  # ---------------------------------------------------------------------------
  #
  def show_peak_heights(self):

    if not self.top.winfo_ismapped():
      self.show_window(1)

    if self.spectra != self.get_spectra():
      self.clear_cb()
      
    if self.spectra:
      for p in sparky.selected_peaks():
        if p.is_assigned:
          self.peak_list.append(self.peak_line(p), p.assignment)
          self.peak_list.listbox.see('end')

  # ---------------------------------------------------------------------------
  #
  def peak_line(self, peak):

    peaks = self.spectrum_peaks(peak)
    line = '%15s ' % peak.assignment
    for p in peaks:
      line = line + '%11.0f ' % self.peak_intensity(p)
    return line

  # ---------------------------------------------------------------------------
  #
  def peak_intensity(self, peak):

    if self.show_volume.get():
      if peak.volume:   return peak.volume
      else:             return 0.0
    if peak.data_height == None:
      return 0.0			# peak groups
    return peak.data_height
    
  # ---------------------------------------------------------------------------
  #
  def spectrum_peaks(self, peak):

    peaks = []
    for s in self.spectra:
      speak = self.get_assigned_peak(s, peak)
      peaks.append(speak)
    return peaks

  # ---------------------------------------------------------------------------
  # Find a peak in the given spectrum matching the given peak's assignment
  # or create one if one does not exist.
  #
  def get_assigned_peak(self, spectrum, p):

    peak = spectrum.find_peak(p.assignment)
    if peak == None:
      peak = sparky.Peak(spectrum, p.frequency)
      copy_assignment(p, peak)
      peak.show_assignment_label()
      if self.center_peaks.get():
        peak.center()
    return peak

# -----------------------------------------------------------------------------
# Copy a peak assignment.
#
def copy_assignment(from_peak, to_peak):

  resonances = from_peak.resonances()
  for a in range(len(resonances)):
    r = resonances[a]
    to_peak.assign(a, r.group.name, r.atom.name)

# -----------------------------------------------------------------------------
#
def show_peak_heights():
  tkutil.the_dialog(relaxation_dialog).show_peak_heights()
